//---------------------------------------------------------------------------

#ifndef CompetitionMort
#define CompetitionMort
//---------------------------------------------------------------------------
#include "MortalityBase.h"
/**
  Competition-Dependent Mortality - Version 1.0

  This evaluates mortality according to Relative Increment (Actual Diameter Growth(i.e. lgm)/Potential Diameter Growth).
  The probability of mortality is a function of Relative Increment.

  Competition-Dependent Mortality can only be used in conjunction with NCI Growth.

  This class's namestring is "competitionmortshell".  The parameter file call
  string is "competitionmortality".

  Copyright 2003 Charles D. Canham.
  @authors Rasmus Astrup, Marissa LeBlanc and Lora E. Murphy

  <br>Edit history:
  <br>-----------------
  <br>February 10, 2005 - Submitted as beta (RA)
*/
class clCompetitionMort : virtual public clMortalityBase {
//note: need the virtual keyword to avoid base class ambiguity.

 public:

/**
 Constructor.  Sets the namestring.
*/
 clCompetitionMort(clSimManager *p_oSimManager);

/**
  Destructor.
*/
 ~clCompetitionMort();

/**
  Reads in values from the parameter file and makes sure all data needed is
  collected.

  @param p_oDoc Parsed DOM tree of parameter file.
*/
 void DoShellSetup(xercesc::DOMDocument *p_oDoc);

/**
  Calculates mortality according to the competition mortality equation.
  @param p_oTree Tree being evaluated.
  @param fDbh DBH of tree being evaluated.
  @param iSpecies Species of the tree being evaluated.
  @return True if the tree is to die, false if it lives.
*/
 bool DoMort(clTree *p_oTree, const float &fDbh, const short int &iSpecies);

 protected:

 short int **mp_iLgmCodes; /**<data member codes for "lgm" member - species by type*/

 float *mp_fCompMort;		/**<Shape parameter for competition-dependent mortality function. Array sized number of species to which this behavior applies.*/
 float *mp_fCompMortMax;        /**<Parameter for competition-dependent morality function that determines the maximum Relative Increment suject to mortality
 Array sized number of species to which this behavior applies.*/
 float m_fNumberYearsPerTimestep; /**<Number of years per timestep*/


 //Parameters to calculate potential growth as in NCI growth
 float *mp_fXb;           /**<Size effect variance parameter (Xb).
 Array sized number of species to which this behavior applies.*/
 float *mp_fMaxPotentialGrowth; /**<Maximum potential growth in cm/yr. Array sized number of species to which this behavior
 applies.*/
 float *mp_fX0;   /**<Size effect mode parameter (X0). Array sized number of species to which this behavior applies.*/

 short int *mp_iIndexes;  /**<Speeds access to the arrays.*/


/**
 * Queries for the return codes of the "lgm" float data member of a tree.  This
 * data member should have been registered by light.  Return codes are captured
 * in the
 * mp_iLgmCodes array.
 * @throws modelErr if there is no code for any species/type combo which uses
 * this behavior.
*/
  void GetLgmVariableCodes();
};
//---------------------------------------------------------------------------
#endif


